<?php

namespace BT\PaygateBetatransfer\Payment;

use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Mvc\Reply\AbstractReply;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

class Betatransfer extends AbstractProvider
{
    protected $supportedCurrencies = ['RUB', 'USD', 'EUR', 'UAH'];

    public function getTitle(): string
    {
        return 'Betatransfer';
    }

    public function getApiEndpoint(): string
    {
        return 'https://betatransfer.io';
    }

    public function verifyConfig(array &$options, &$errors = []): bool
    {
        if (empty($options['public_key']) || empty($options['secret_key'])) {
            $errors[] = XF::phrase('bt_betatransfer_you_must_provide_all_data');
        }

        if (!$errors) {
            return true;
        }

        return false;
    }

    public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase): AbstractReply
    {
        $paymentProfile = $purchase->paymentProfile;

        $payment = $this->getPaymentParams($purchaseRequest, $purchase);
        $action  = $this->getApiEndpoint() . '/api/payment?token=' . $paymentProfile->options['public_key'];

        $response = XF::app()->http()->client()
            ->post($action, [
                'headers'     => [
                    'Content-Type' => 'application/x-www-form-urlencoded'
                ],
                'form_params' => $payment,
                'exceptions'  => false
            ]);

        if ($response) {
            $responseData = json_decode($response->getBody()->getContents(), true);

            if (!empty($responseData['urlPayment'])) {
                return $controller->redirect($responseData['urlPayment']);
            }
        }

        return $controller->error(XF::phrase('something_went_wrong_please_try_again'));
    }

    public function setupCallback(Request $request): CallbackState
    {
        $state = new CallbackState();
        $state->transactionId = $request->filter('id', 'uint');
        $state->requestKey = $request->filter('orderId', 'str');
        $state->costAmount = $request->filter('amount', 'str');
        $state->costCurrency = $request->filter('currency', 'str');
        $state->status = $request->filter('status', 'str');
        $state->signature = $request->filter('sign', 'str');

        $state->input = array_merge($_POST, [
            'callback_ip' => $request->getIp(),
        ]);

        return $state;
    }

    public function validatePurchasableData(CallbackState $state): bool
    {
        $paymentProfile = $state->getPaymentProfile();
        $options = $paymentProfile->options;

        if (empty($options['secret_key'])) {
            $state->logType = 'error';
            $state->logMessage = 'Invalid secret_key.';

            return false;
        }

        $sign = $this->generateSign(
            [$state->costAmount, $state->requestKey],
            $options['secret_key']
        );

        if (hash_equals($state->signature, $sign)) {
            return true;
        }

        $state->logType = 'error';
        $state->logMessage = 'Invalid sign!';

        return false;
    }

    public function validateCost(CallbackState $state): bool
    {
        $purchaseRequest = $state->getPurchaseRequest();

        if (
            round($state->costAmount, 2) == round($purchaseRequest->cost_amount, 2)
            && $state->costCurrency == $purchaseRequest->cost_currency
        ) {
            return true;
        }

        $state->logType = 'error';
        $state->logMessage = 'Invalid cost amount or cost currency';

        return false;
    }

    public function getPaymentResult(CallbackState $state): void
    {
        if (in_array($state->status, ['success', 'partial_payment'])) {
            $state->paymentResult = CallbackState::PAYMENT_RECEIVED;
        }
    }

    public function prepareLogData(CallbackState $state): void
    {
        $state->logDetails = $state->input ?? [];
    }

    public function supportsRecurring(PaymentProfile $paymentProfile, $unit, $amount, &$result = self::ERR_NO_RECURRING): bool
    {
        $result = self::ERR_NO_RECURRING;

        return false;
    }

    public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode): bool
    {
        return in_array($currencyCode, $this->supportedCurrencies);
    }

    protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase): array
    {
        $paymentProfile = $purchase->paymentProfile;

        $params['amount'] = $purchase->cost;
        $params['currency'] = $purchase->currency;
        $params['orderId'] = $purchaseRequest->request_key;
        $params['urlSuccess'] = $purchase->returnUrl;
        $params['urlFail'] = $purchase->cancelUrl;
        $params['urlResult'] = $this->getCallbackUrl();
        $params['payerId'] = $purchaseRequest->user_id;
        $params['fullCallback'] = 1;
        $params['paymentSystem'] = $paymentProfile->options['payment_system'];
        $params['sign'] = $this->generateSign(array_filter($params), $paymentProfile->options['secret_key']);

        return array_filter($params);
    }

    protected function getSupportedRecurrenceRanges(): array
    {
        return [];
    }

    private function generateSign(array $data, string $secret): string
    {
        return md5(implode('', $data) . $secret);
    }
}